rm(list=ls())
getwd()
setwd("") # SET YOUR WORKING DIRECTORY #

setwd("C:/Users/Pedro/Documents/Classes/Methods in Ecology/2013 fall") # May have to adapt that


## PART I. REGRESSION ON UNTRANSFORMED DATA

## read the area-species data from file
orig_data <- read.table("Regression_Data.txt", header=T)
attach(orig_data)

## draw a scatterplot of the untransformed data
plot(Area, Nspecies)

## define a simple linear regression model relating Nspecies to Area
orig_model <- lm(Nspecies ~ Area)
summary(orig_model)

## create new variables to hold the fitted values (Y_hats) and the residuals
Y_hats <- fitted(orig_model)
residuals <- resid(orig_model)

## draw a scatterplot of the residuals vs. the fitted values
Y_hats <- fitted(orig_model)
residuals <- resid(orig_model)

## PART II. REGRESSION ON LOG-TRANSFORMED DATA
par(mfrow=c(1,3))

## transform Area and Nspecies by taking log of each variable
log_Area <- log(Area)
log_Nspecies <- log(Nspecies)
plot(log_Area, log_Nspecies,ylim=c(1,8))

## define a simple linear regression model relating log_Nspecies to log_Area
trans_model <- lm(log_Nspecies ~ log_Area)
summary(trans_model)

## check our new residuals vs. the fitted values plot
Y_hats <- fitted(trans_model)
residuals <- resid(trans_model)
plot(Y_hats, residuals)
abline(h=0, col="red")

## PART III. CONFIDENCE INTERVALS

## create a matrix with the estimated coefficients and their standard errors 
coeffs <- summary(trans_model)$coefficients

## retrieve critical value from the t-distribution with n-2 degrees of freedom
t_crit <- qt(0.975, length(log_Area)-2)

## create 95% confidence interval for B0_hat (the intercept)
B0_hat <- coeffs[1,1]
B0_se <- coeffs[1,2]
B0_lower <- B0_hat - (B0_se * t_crit)
B0_upper <- B0_hat + (B0_se * t_crit)
cat("95% CI for B0: [", B0_lower, ", ", B0_upper, "]\n", sep="")

## create 95% confidence interval for B1_hat (the slope)
B1_hat <- coeffs[2,1]
B1_se <- coeffs[2,2]
B1_lower <- B1_hat - (B1_se * t_crit)
B1_upper <- B1_hat + (B1_se * t_crit)
cat("95% CI for B1: [", B1_lower, ", ", B1_upper, "]\n", sep="")

## create a new data frame to hold the confidence and prediction intervals
pred_frame <- data.frame(log_Area)

## generate the 95% confidence and prediction intervals
conf_intervals <- predict(trans_model, int="c", newdata=pred_frame)
pred_intervals <- predict(trans_model, int="p", newdata=pred_frame)


## plot the log-transformed data, the regression line, and the intervals
plot(log_Area, log_Nspecies, ylim=range(log_Nspecies, pred_intervals))
matlines(pred_frame$log_Area, pred_intervals, lty=c(1,3,3), col="red")
matlines(pred_frame$log_Area, conf_intervals, lty=c(1,2,2), col="blue")
par(mfrow=c(1,1))

## PART IV. REGRESSION WITH A BAYESIAN APPROACH

library(R2OpenBUGS)
n <- NROW(orig_data)
x <- log_Area
y <-log_Nspecies

### Analysis using WinBUGS
# Write model
linreg<-function(){
  for (i in 1:n)			      # for each of the sites
  {
    y[i] ~ dnorm(mean[i], prec)	# assume normal distribution
    mean[i] <- a + b*x[i]
  }
  # uninformative priors
  a ~ dnorm(0, 1.0E-6)
  b ~ dnorm(0, 1.0E-6)
  prec ~ dgamma(0.001, 0.001)
  for (j in 1:10)
  {
    pred[j] <- a + b*(j)
  }
}
write.model(linreg, "linreg.txt")

# Bundle data
win.data <- list("x","y","n")

# Inits function
inits <- function(){ list(a=runif(1), b=runif(1) ,prec = 10)}

# Parameters to estimate
params <- c("a","b","prec","pred")

# MCMC settings
nc = 1 
ni=40000
nb=400
nt=20

# Start Gibbs sampler
out <- bugs(data = win.data, inits = inits, parameters = params, model = "linreg.txt",
n.thin = nt, n.chains = nc, n.burnin = nb, n.iter = ni, codaPkg=T)

library(coda)
reg.coda<-read.bugs(out)

##Explore the results
summary(reg.coda)

##Explore the related plots
densityplot(reg.coda[,1], xlab="Intercept(a)")
densityplot(reg.coda[,2], xlab="Slope(b)")

###Explore the autocorrelation plots
autocorr.plot(reg.coda[,1])
autocorr.plot(reg.coda[,2])

##Plot the fitted model together with the credibility envelope and the observed values
results<-summary(reg.coda)
plot(log_Area, log_Nspecies, ylim=range(log_Nspecies, pred_intervals))
for(i in 1:1000){
abline(as.numeric(reg.coda[i,1]),as.numeric(reg.coda[i,2]),col="lightgray")}
abline(results$statistics[1,1],results$statistics[2,1])

## detach the data
detach(orig_data)
